import org.jetbrains.kotlin.gradle.tasks.KotlinCompile


plugins {
	id("org.springframework.boot") version "2.4.4"
	id("io.spring.dependency-management") version "1.0.11.RELEASE"
	kotlin("jvm") version "1.6.20"
	kotlin("plugin.spring") version "1.6.20"
	kotlin("plugin.jpa") version "1.6.20"
	kotlin("plugin.serialization") version "1.6.20"
}

group = "com.aitrainer"
version = "1.0.57"
java.sourceCompatibility = JavaVersion.VERSION_1_8

repositories {
	mavenCentral()
}

dependencies {
	implementation("org.springframework.boot:spring-boot-starter-data-jpa")
	implementation("org.springframework.boot:spring-boot-starter-web")
	implementation("org.springframework.boot:spring-boot-starter-aop")
	implementation("org.springframework.boot:spring-boot-starter-validation")
	implementation("org.springframework.boot:spring-boot-starter-security")
	implementation("org.springframework.security.oauth.boot:spring-security-oauth2-autoconfigure:2.4.4")
	implementation("org.springframework.security.oauth:spring-security-oauth2:2.5.0.RELEASE")
	implementation("com.fasterxml.jackson.module:jackson-module-kotlin")
	implementation("org.jetbrains.kotlin:kotlin-stdlib-jdk8")
	implementation("org.apache.logging.log4j:log4j-core:2.14.0")
	implementation("org.apache.logging.log4j:log4j-api:2.14.0")
	implementation("org.slf4j:slf4j-api:1.7.36")
	implementation("org.jetbrains.kotlinx:kotlinx-serialization-runtime:1.0-M1-1.4.0-rc") // JVM dependency
	implementation("io.jsonwebtoken:jjwt:0.9.1")
	implementation("org.yaml:snakeyaml:1.30")
	implementation("com.google.code.gson:gson:2.9.0")
	implementation("org.projectlombok:lombok:1.18.22")
	implementation("org.jetbrains.kotlin:kotlin-reflect:1.6.20")
	implementation("com.github.ulisesbocchio:jasypt-spring-boot-starter:3.0.4")

	runtimeOnly("mysql:mysql-connector-java")
	testImplementation("org.springframework.boot:spring-boot-starter-test") {
		exclude(group = "org.junit.vintage", module = "junit-vintage-engine")
	}
	testImplementation("junit:junit:4.13.2")
	testImplementation("org.jetbrains.kotlin:kotlin-test-junit5:1.6.20")

}

tasks.withType<Test> {
	useJUnitPlatform()
}

tasks.withType<KotlinCompile> {
	kotlinOptions {
		freeCompilerArgs = listOf("-Xjsr305=strict", "-opt-in=kotlin.RequiresOptIn")
		jvmTarget = "1.8"
	}
}
