package com.aitrainer.api.security

import com.aitrainer.api.service.UserDetailsServiceImpl
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.security.authentication.AuthenticationManager
import org.springframework.security.authentication.BadCredentialsException
import org.springframework.security.authentication.DisabledException
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.web.bind.annotation.*
import org.springframework.stereotype.Component
import javax.servlet.http.HttpServletRequest


@Component
@RestController
//@CrossOrigin
@RequestMapping("/api")
class JwtAuthenticationController {
    @Autowired
    private val authenticationManager: AuthenticationManager? = null

    @Autowired
    private val jwtTokenUtil: JwtTokenUtil? = null

    @Autowired
    private val jwtUserDetailsService: UserDetailsServiceImpl? = null

    @PostMapping("/authenticate")
    fun generateAuthenticationToken(@RequestBody authenticationRequest: JwtRequest, request: HttpServletRequest): ResponseEntity<*> {

        authenticate(authenticationRequest.username!!, authenticationRequest.password!!)
        println("authenticating from: ${request.remoteAddr}")

        val userDetails = jwtUserDetailsService
                ?.loadUserByUsername(authenticationRequest.username)
        val token: String = jwtTokenUtil!!.generateToken(userDetails!!)
        return ResponseEntity.ok<Any>(JwtResponse(token))
    }

    private fun authenticate(username: String, password: String) {
        try {
            authenticationManager!!.authenticate(UsernamePasswordAuthenticationToken(username, password))
        } catch (e: DisabledException) {
            throw Exception("USER_DISABLED", e)
        } catch (e: BadCredentialsException) {
            throw Exception("INVALID_CREDENTIALS", e)
        } catch( e: Exception) {
            println("Auth exception " + e.message)
            throw Exception("Other Exception", e)
        }
    }
}
